import os
import glob
from catsndogs.data import training_dir, is_cat, is_dog

dogs = os.path.join(training_dir, "dog")
cats = os.path.join(training_dir, "cat")

def plot_samples(axes=None,
                 n=4):
    import matplotlib.pyplot as plt
    import numpy as np
    from skimage import io

    m = 2
    cats = glob.glob(os.path.join(training_dir, "cat", "*.jpg"))
    dogs = glob.glob(os.path.join(training_dir, "dog", "*.jpg"))

    if axes is None:
        f, axes = plt.subplots(m, n, figsize=(n*2, m*2))
    else:
        f = axes.figure()

    for i in range(n):
        ax = axes[0, i]
        img = io.imread(np.random.choice(cats))
        ax.imshow(img)
        ax.set_title("({}) A cat".format(chr(ord("a") + i)), loc="left")
        ax.set_xticks([])
        ax.set_yticks([])

    for i in range(n):
        ax = axes[1, i]
        img = io.imread(np.random.choice(dogs))
        ax.imshow(img)
        ax.set_title("({}) A dog".format(chr(ord("e") + i)), loc="left")
        ax.set_xticks([])
        ax.set_yticks([])

    plt.tight_layout()



import os
import glob
from catsndogs.data import get_training_data
folder = get_training_data()
cats = glob.glob(os.path.join(get_training_data(), "cat", "*.jpg"))
dogs = glob.glob(os.path.join(get_training_data(), "dog", "*.jpg"))



import catsndogs.test
import catsndogs.training
import pytest
import glob
import os

def test_test_data():
    folders = glob.glob(os.path.join(catsndogs.test.folder, "*"))
    assert(len(folders) == 2)
    assert(len(catsndogs.test.cats) == 239)
    assert(len(catsndogs.test.dogs) == 500)

def test_training_data_data():
    folders = glob.glob(os.path.join(catsndogs.training.folder, "*"))
    assert(len(folders) == 2)
    assert(len(catsndogs.training.cats) == 949)
    assert(len(catsndogs.training.dogs) == 1998)





from appdirs import AppDirs
import requests
import tarfile
import os

app_dirs = AppDirs("catsndogs", "simonpf")
training_dir = os.path.join(app_dirs.user_data_dir, "training_data")
test_dir = os.path.join(app_dirs.user_data_dir, "test_data")

def is_cat(file):
    """
    Args:
        Image filename.
    Returns:
        Whether the image contains a cat.

    """
    return os.path.basename(file)[0].isupper()

def is_dog(file):
    """
    Args:
        Image filename.
    Returns:
        Whether the image contains a dog.
    """
    return not is_cat(file)

def download_training_data():
    """
    Download and extract  training data and store to the user's local data directory.
    """
    filename = os.path.join(app_dirs.user_data_dir, "training_data.tar.gz")

    if not os.path.exists(app_dirs.user_data_dir):
        os.makedirs(app_dirs.user_data_dir)

    # Download data
    training_url = "http://spfrnd.de/datasets/catsndogs/training_data.tar.gz"
    r = requests.get(training_url)
    with open(filename, "wb") as ds:
        for chunk in r.iter_content(chunk_size=128):
            ds.write(chunk)

    # Extract
    tar = tarfile.open(filename)
    tar.extractall(path=app_dirs.user_data_dir)
    tar.close()

    os.remove(filename)

def download_test_data():
    """
    Download and extract  test data and store to the user's local data directory.
    """
    filename = os.path.join(app_dirs.user_data_dir, "test_data.tar.gz")

    if not os.path.exists(app_dirs.user_data_dir):
        os.makedirs(app_dirs.user_data_dir)

    # Download data
    test_url = "http://spfrnd.de/datasets/catsndogs/test_data.tar.gz"
    r = requests.get(test_url)
    with open(filename, "wb") as ds:
        for chunk in r.iter_content(chunk_size=128):
            ds.write(chunk)

    # Extract
    tar = tarfile.open(filename)
    tar.extractall(path=app_dirs.user_data_dir)
    tar.close()

    os.remove(filename)

def get_training_data():
    """
    If necessary, downloads training data and
    returns directory containing training data.
    """

    if not os.path.exists(training_dir):
        download_training_data()
    return training_dir

def get_test_data():
    """
    If necessary, downloads test data and
    returns directory containing test data.
    """
    if not os.path.exists(test_dir):
        download_test_data()
    return test_dir



"""
This sub-module implements functionality to process the original
Oxford-IIIT Pet Dataset and produce a dataset with smaller and centered
images.
"""
import glob
import os
import re
import shutil
import xmltodict
import PIL
import PIL.Image

################################################################################
# Extract images from Oxford dataset.
################################################################################

def extract_bounding_box(annotation):
    """
    Extract bounding box from annotation file.

    Args:
        annotation: Filename of the .xml file cotaining the annotations
    Return:
        Tuple (xmin, ymin, xmax, ymax) representing the bounding box around
        the cat or dog.
    """
    dct = xmltodict.parse(open(annotation, "r").read())

    obj = dct["annotation"]["object"]
    if type(obj) == list:
        obj = obj[0]

    xmin = obj["bndbox"]["xmin"]
    ymin = obj["bndbox"]["ymin"]
    xmax = obj["bndbox"]["xmax"]
    ymax = obj["bndbox"]["ymax"]
    return (int(xmin), int(ymin), int(xmax), int(ymax))

class Image:
    """
    The Image class represents an image from the original dataset
    combined with the bounding box around the head of the animal.
    """
    def __init__(self,
                 image,
                 annotation):
        """
        Args:
            image: Filename of the image file
            annotation: Filename of the corresponding annotation
                file.
        """
        self.filename = image
        self.bounding_box = extract_bounding_box(annotation)

    @property
    def data(self):
        """
        Return image as PIL.Image
        """
        return PIL.Image.open(self.filename)

    @property
    def roi(self):
        """
        Return image centered around bounding box.
        """
        image = PIL.Image.open(self.filename)
        w, h = image.size
        xmin, ymin, xmax, ymax = self.bounding_box
        dx = min(xmin, w - xmax)
        dy = min(ymin, h - ymax)
        d = min(dx, dy)
        return image.crop((xmin - d, ymin -d, xmax + d, ymax + d))

    @property
    def roi_resized(self):
        """
        Centered and resized image.
        """
        roi = self.roi
        return roi.resize((128, 128))

class RawData:
    """
    Class representing the data of the original dataset. Provides
    access to annotated images in the dataset.
    """
    def __init__(self, path):
        self.image_path = os.path.join(path, "images")
        self.images = glob.glob(os.path.join(self.image_path, "*.jpg"))
        self.images.sort()
        self.annotation_path = os.path.join(path, "annotations", "xmls")
        self.annotations = glob.glob(os.path.join(self.annotation_path, "*xml"))
        self.annotations.sort()

    def __len__(self):
        """
        Number of available annotations.
        """
        return(len(self.annotations))

    def __getitem__(self, index):
        """
        Get image from dataset.
        """
        index = index % len(self)
        annotation = self.annotations[index]
        name, _ = os.path.splitext(os.path.basename(annotation))
        image = os.path.join(self.image_path, name + ".jpg")
        return Image(image, annotation)

################################################################################
# Splitting data
################################################################################
filename_pattern = re.compile("([\S]*)_([\d]*).jpg")

def read_classes(input_folder):
    """
    Read images from folders and sort into dict according to classes.

    Args:
        input_folder: Directory containing the Oxford-III Pet images.

    Returns: Dict with class names as keys and list of filenames of
        corresponding images as entries.
    """
    classes = {}
    files = glob.glob(os.path.join(os.path.expanduser(input_folder), "*.jpg"))
    for f in files:
        match = filename_pattern.match(os.path.basename(f))
        c = match.group(1)
        if not c in classes:
            classes[c] = []
        classes[c] += [f]
    return classes


def split_data(input_folder, training_folder, test_folder, split=0.8):
    """
    Split data sequentially into training and data.

    Args:
        input_folder: The folder containing the input images.
        training_folder: Folder in which to store the training images.
        test_folder: Folder in which to store the test images.
        split: Ratio between the number of images used for training
            and those used for testing.
    """
    training_folder = os.path.expanduser(training_folder)
    test_folder = os.path.expanduser(test_folder)

    if not os.path.exists(training_folder):
        try:
            os.makedirs(training_folder)
            os.makedirs(os.path.join(training_folder, "cat"))
            os.makedirs(os.path.join(training_folder, "dog"))
        except:
            pass

    if not os.path.exists(test_folder):
        try:
            os.makedirs(test_folder)
            os.makedirs(os.path.join(test_folder, "cat"))
            os.makedirs(os.path.join(test_folder, "dog"))
        except:
            pass

    classes = read_classes(input_folder)
    for c in classes:
        files = classes[c]
        if c[0].isupper():
            f_train = os.path.join(training_folder, "cat")
            f_test = os.path.join(test_folder, "cat")
        else:
            f_train = os.path.join(training_folder, "dog")
            f_test = os.path.join(test_folder, "dog")

        n = int(split * len(files))
        for f in files[:n]:
            out = os.path.join(f_train, os.path.basename(f))
            shutil.copyfile(f, out)
        for f in files[n:]:
            out = os.path.join(f_test, os.path.basename(f))
            shutil.copyfile(f, out)



import torch
import os
import glob
from torch.utils.data import Dataset
from catsndogs.data import get_training_data, get_test_data
from skimage import io

class CatsNDogsData():
    def __init__(self, mode="training"):
        if mode == "training":
            self.data_dir = get_training_data()
        else:
            self.data_dir = get_test_data()

        self._load_data()

    def _load_data(self):
        self.files = glob.glob(os.path.join(self.data_dir, "*.jpg"))
        self.classes = [int(os.path.basename(f)[0].isupper()) for f in self.files]

        self.x = torch.zeros(len(self.files), 3, 64, 64)
        self.y = torch.zeros(len(self.files), 1)
        for i, (f, c) in enumerate(zip(self.files, self.classes)):
            self.x.data[i, :, :] = torch.tensor(io.imread(f)).float().transpose(0, -1) / 256.0
            self.y.data[i, 0] = c
        self.x.data = -1.0 + 2.0 * self.x.data

    def __len__(self):
        return self.x.size()[0]

    def __getitem__(self, index):
        return (self.x[index], self.y[index])

training_data = CatsNDogsData("training")
test_data = CatsNDogsData("test")
